// Protocol V2 Types and Interfaces
// This file contains all type definitions for the improved TAP protocol

export interface ITestMetadata {
  // Timing
  time?: number;              // milliseconds
  startTime?: number;         // Unix timestamp
  endTime?: number;          // Unix timestamp
  
  // Status
  skip?: string;             // skip reason
  todo?: string;             // todo reason
  retry?: number;            // retry attempt
  maxRetries?: number;       // max retries allowed
  
  // Error details
  error?: {
    message: string;
    stack?: string;
    diff?: string;
    actual?: any;
    expected?: any;
    code?: string;
  };
  
  // Test context
  file?: string;             // source file
  line?: number;             // line number
  column?: number;           // column number
  
  // Custom data
  tags?: string[];           // test tags
  custom?: Record<string, any>;
}

export interface ITestResult {
  ok: boolean;
  testNumber: number;
  description: string;
  directive?: {
    type: 'skip' | 'todo';
    reason?: string;
  };
  metadata?: ITestMetadata;
}

export interface IPlanLine {
  start: number;
  end: number;
  skipAll?: string;
}

export interface IProtocolMessage {
  type: 'test' | 'plan' | 'comment' | 'version' | 'bailout' | 'protocol' | 'snapshot' | 'error' | 'event';
  content: any;
}

export interface IProtocolVersion {
  version: string;
  features?: string[];
}

export interface ISnapshotData {
  name: string;
  content: any;
  format?: 'json' | 'text' | 'binary';
}

export interface IErrorBlock {
  testNumber?: number;
  error: {
    message: string;
    stack?: string;
    diff?: string;
    actual?: any;
    expected?: any;
  };
}

// Enhanced Communication Types
export type EventType = 
  | 'test:queued'
  | 'test:started' 
  | 'test:progress'
  | 'test:completed'
  | 'suite:started'
  | 'suite:completed'
  | 'hook:started'
  | 'hook:completed'
  | 'assertion:failed';

export interface ITestEvent {
  eventType: EventType;
  timestamp: number;
  data: {
    testNumber?: number;
    description?: string;
    suiteName?: string;
    hookName?: string;
    progress?: number; // 0-100
    duration?: number;
    error?: IEnhancedError;
    [key: string]: any;
  };
}

export interface IEnhancedError {
  message: string;
  stack?: string;
  diff?: IDiffResult;
  actual?: any;
  expected?: any;
  code?: string;
  type?: 'assertion' | 'timeout' | 'uncaught' | 'syntax' | 'runtime';
}

export interface IDiffResult {
  type: 'string' | 'object' | 'array' | 'primitive';
  changes: IDiffChange[];
  context?: number; // lines of context
}

export interface IDiffChange {
  type: 'add' | 'remove' | 'modify';
  path?: string[]; // for object/array diffs
  oldValue?: any;
  newValue?: any;
  line?: number; // for string diffs
  content?: string;
}

// Protocol markers
export const PROTOCOL_MARKERS = {
  START: '⟦TSTEST:',
  END: '⟧',
  META_PREFIX: 'META:',
  ERROR_PREFIX: 'ERROR',
  ERROR_END: '/ERROR',
  SNAPSHOT_PREFIX: 'SNAPSHOT:',
  SNAPSHOT_END: '/SNAPSHOT',
  PROTOCOL_PREFIX: 'PROTOCOL:',
  SKIP_PREFIX: 'SKIP:',
  TODO_PREFIX: 'TODO:',
  EVENT_PREFIX: 'EVENT:',
} as const;

// Protocol version
export const PROTOCOL_VERSION = '2.0.0';